#!/usr/bin/env python3
# -*- coding: utf-8 -*-

extensions = [
	'sphinx.ext.autodoc',
	'jaraco.packaging.sphinx',
	'rst.linker',
]

master_doc = 'index'

link_files = {
	'../CHANGES.rst': dict(
		using=dict(
			GH='https://github.com',
		),
		replace=[
			dict(
				pattern=r'(Issue #|\B#)(?P<issue>\d+)',
				url='{package_url}/issues/{issue}',
			),
			dict(
				pattern=r'^(?m)((?P<scm_version>v?\d+(\.\d+){1,2}))\n[-=]+\n',
				with_scm='{text}\n{rev[timestamp]:%d %b %Y}\n',
			),
			dict(
				pattern=r'PEP[- ](?P<pep_number>\d+)',
				url='https://www.python.org/dev/peps/pep-{pep_number:0>4}/',
			),
			dict(
				pattern=r"pip #(?P<pip_issue>\d+)",
				url='{GH}/pypa/pip/issues/{pip_issue}',
			),
		],
	),
}



import textwrap
import os
import subprocess
import sys
import signal
import glob
import itertools


class PathReader:
	@staticmethod
	def _read_file(filename):
		root = os.path.dirname(filename)
		return (
			os.path.join(root, path.rstrip())
			for path in open(filename)
			if path.strip()
			and not path.startswith('#')
			and not path.startswith('import ')
		)

	@classmethod
	def _read(cls, target):
		"""
		As .pth files aren't honored except in site dirs,
		read the paths indicated by them.
		"""
		pth_files = glob.glob(os.path.join(target, '*.pth'))
		file_items = map(cls._read_file, pth_files)
		return itertools.chain.from_iterable(file_items)


def _inject_sitecustomize(target):
	"""
	Create a sitecustomize file in the target that will install
	the target as a sitedir.

	Only needed on Python 3.2 and earlier to workaround #1.
	"""
	if sys.version_info > (3, 3):
		return

	hook = textwrap.dedent("""
		import site
		site.addsitedir({target!r})
		""").lstrip().format(**locals())
	sc_fn = os.path.join(target, 'sitecustomize.py')
	with open(sc_fn, 'w') as strm:
		strm.write(hook)


def _build_env(target):
	"""
	Prepend target and .pth references in target to PYTHONPATH
	"""
	env = dict(os.environ)
	suffix = env.get('PYTHONPATH')
	prefix = target,
	items = itertools.chain(
		prefix,
		PathReader._read(target),
		(suffix,) if suffix else (),
	)
	joined = os.pathsep.join(items)
	env['PYTHONPATH'] = joined
	return env


def _setup_env(target):
	_inject_sitecustomize(target)
	return _build_env(target)


def with_path(target, params):
	"""
	Launch Python with target on the path and params
	"""
	def null_handler(signum, frame):
		pass

	signal.signal(signal.SIGINT, null_handler)
	cmd = [sys.executable] + params
	return subprocess.Popen(cmd, env=_setup_env(target)).wait()


def with_path_overlay(target, params):
	"""
	Overlay Python with target on the path and params
	"""
	cmd = [sys.executable] + params
	os.execve(sys.executable, cmd, _setup_env(target))



import copy

import pkg_resources

from rwt import deps


def test_entry_points():
	"""
	Ensure entry points are visible after making packages visible
	"""
	with deps.on_sys_path('jaraco.mongodb'):
		eps = pkg_resources.iter_entry_points('pytest11')
		assert list(eps), "Entry points not found"


class TestInstallCheck:
	def test_installed(self):
		assert deps.pkg_installed('rwt')

	def test_not_installed(self):
		assert not deps.pkg_installed('not_a_package')

	def test_installed_version(self):
		assert not deps.pkg_installed('rwt==0.0')

	def test_not_installed_args(self):
		args = [
			'-i', 'https://devpi.net',
			'-r', 'requirements.txt',
			'rwt',
			'not_a_package',
			'rwt==0.0',
		]
		expected = copy.copy(args)
		expected.remove('rwt')
		filtered = deps.not_installed(args)
		assert list(filtered) == expected


class TestLoad:
	def test_no_args_passes(self):
		"""
		If called with no arguments, load() should still provide
		a context.
		"""
		with deps.load():
			pass



"""
This script demonstrates how RWT facilitates the
simple execution of complex tasks with their
dependencies.

Run this example with ``rwt -- $script``.

It creates a MongoDB instance, and then runs some
assertions against it.

As it uses `jaraco.mongodb
<https://pypi.org/project/jaraco.mongodb>`_, MongoDB
must be installed to a typical location or
available on PATH; set MONGODB_HOME to
specify the MongoDB version to use for the ephemeral
instance.

Running this script with RWT leaves no trace of its
execution, other than adding packages to the pip
cache (if available), so feel free to give it a try.

This script gets read three times:

- First ``rwt`` reads it statically to determine the
  dependencies found in ``__requires__`` and
  installs those dependencies in a temp dir.
- Then, the script is executed by a Python interpreter
  which executes the script up to the __main__ block,
  which invokes pytest with the script name as the
  first argument.
- ``py.test`` then runs the test, using the plugins
  from jaraco.mongodb to satisfy the ``mongodb_instance``
  fixture for the test.

Alternately, the script could be
invoked thus:  ``rwt -- -m pytest $script``, which would
eliminate the need for the ``__main__`` block and
would skip the second processing of the script.
"""

__requires__ = [
	'pytest',
	'jaraco.mongodb>=3.10',
]

if __name__ == '__main__':
	# invoke pytest on this script
	import pytest
	import sys
	sys.exit(pytest.main(sys.argv))

import random
import itertools

import pytest
from jaraco.mongodb.testing import assert_covered


@pytest.fixture
def docs_in_db(mongodb_instance):
	"""
	Install 100 records with random numbers
	"""
	conn = mongodb_instance.get_connection()
	coll = conn.test_db.test_coll
	coll.drop()
	coll.create_index('number')
	n_records = 100
	for n in itertools.islice(itertools.count(), n_records):
		doc = dict(
			number=random.randint(0, 2**32-1),
			value='some value',
		)
		conn.test_db.test_coll.insert(doc)
	return coll


def test_records(docs_in_db, mongodb_instance):
	"Test 100 records are present and query is covered"
	# load all the numbers to ensure the index is in RAM
	_hint = 'number_1'
	_filter = {'number': {'$gt': 0}}
	_projection = {'number': True, '_id': False}
	cur = docs_in_db.find(filter=_filter, projection=_projection).hint(_hint)
	assert_covered(cur)

	# consume the cursor for good measure
	docs = list(cur)
	assert len(docs) == 100



"""
Emit parameters to pip extracted from a script.
"""

import sys

from .scripts import DepsReader


def run():
	script, = sys.argv[1:]
	deps = DepsReader.load(script).read()
	print(' '.join(deps.params()))


__name__ == '__main__' and run()



"""
Upgrade to pip 10 before installing requirements, working
around error reported in tox-dev/tox#786
"""

import sys
import subprocess
import shlex


def main():
	subprocess.check_call(shlex.split(
		'python -m pip install pip>=10'
	))
	subprocess.check_call(shlex.split(
		'python -m pip install') + sys.argv[1:])


__name__ == '__main__' and main()



from __future__ import print_function

import os
import sys
import contextlib
import subprocess
import tempfile
import shutil
import itertools
import functools

try:
	from pip._vendor import pkg_resources
except ImportError:
	import pkg_resources


filterfalse = getattr(itertools, 'filterfalse', None) or itertools.ifilterfalse


@contextlib.contextmanager
def _update_working_set():
	"""
	Update the master working_set to include these new packages.

	TODO: would be better to use an officially-supported API,
	but no suitable API is apparent.
	"""
	try:
		pkg_resources = sys.modules['pkg_resources']
		pkg_resources._initialize_master_working_set()
	except KeyError:
		# it's unnecessary to re-initialize when it hasn't
		# yet been initialized.
		pass
	yield


@contextlib.contextmanager
def load(*args):
	target = tempfile.mkdtemp(prefix='rwt-')
	cmd = (
		sys.executable,
		'-m', 'pip',
		'install',
		'-t', target,
	) + args
	with _patch_prefix():
		args and subprocess.check_call(cmd)
	try:
		yield target
	finally:
		shutil.rmtree(target)


@contextlib.contextmanager
def load_direct(*args):
	target = tempfile.mkdtemp(prefix='rwt-')
	args = ('install', '-t', target) + args
	with _patch_prefix():
		__import__('pip').main(list(args))
	try:
		yield target
	finally:
		shutil.rmtree(target)


@contextlib.contextmanager
def _patch_prefix():
	"""
	To workaround pypa/pip#4106, override the system prefix with
	a user prefix, restoring the original file after.
	"""
	cfg_fn = os.path.expanduser('~/.pydistutils.cfg')
	with _save_file(cfg_fn):
		with open(cfg_fn, 'w') as cfg:
			cfg.write('[install]\nprefix=\n')
		yield


@contextlib.contextmanager
def _save_file(filename):
	"""
	Capture the state of filename and restore it after the context
	exits.
	"""
	# For now, only supports a missing filename.
	if os.path.exists(filename):
		tmpl = "Unsupported with extant {filename}"
		raise NotImplementedError(tmpl.format(**locals()))
	try:
		yield
	finally:
		if os.path.exists(filename):
			os.remove(filename)


@contextlib.contextmanager
def on_sys_path(*args):
	"""
	Install dependencies via args to pip and ensure they have precedence
	on sys.path.
	"""
	with load(*args) as target:
		sys.path.insert(0, target)
		try:
			with _update_working_set():
				yield target
		finally:
			sys.path.remove(target)


def pkg_installed(spec):
	try:
		pkg_resources.require(spec)
	except Exception:
		return False
	return True


not_installed = functools.partial(filterfalse, pkg_installed)



#!/usr/bin/env python

# Project skeleton maintained at https://github.com/jaraco/skeleton

import setuptools

params = dict(
	use_scm_version=True,
)
if __name__ == '__main__':
	setuptools.setup(**params)



import os
import sys
import ast
import tokenize
import itertools
import io


try:
	from pip._vendor import pkg_resources
except ImportError:
	import pkg_resources


if sys.version_info < (3,):
	filter = itertools.ifilter
	map = itertools.imap


class Dependencies(list):
	index_url = None
	dependency_links = []

	def params(self):
		prefix = ['--index-url', self.index_url] if self.index_url else []
		for link in self.dependency_links:
			prefix.extend(['--find-links', link])
		return prefix + self


class DepsReader:
	"""
	Given a Python script, read the dependencies it declares.
	Does not execute the script, so expects __requires__ to be
	assigned a static list of strings.
	"""
	def __init__(self, script):
		self.script = script

	@classmethod
	def load(cls, script_path):
		with io.open(script_path) as stream:
			return cls(stream.read())

	@classmethod
	def try_read(cls, script_path):
		"""
		Attempt to load the dependencies from the script,
		but return an empty list if unsuccessful.
		"""
		try:
			reader = cls.load(script_path)
			return reader.read()
		except Exception:
			return Dependencies()

	@classmethod
	def search(cls, params):
		"""
		Given a (possibly-empty) series of parameters to a
		Python interpreter, return any dependencies discovered
		in a script indicated in the parameters. Only honor the
		first file found.
		"""
		files = filter(os.path.isfile, params)
		return cls.try_read(next(files, None)).params()

	def read(self):
		"""
		>>> DepsReader("__requires__=['foo']").read()
		['foo']
		"""
		reqs_raw = self._read('__requires__')
		strings = map(str, pkg_resources.parse_requirements(reqs_raw))
		deps = Dependencies(strings)
		try:
			deps.index_url = self._read('__index_url__')
		except Exception:
			pass
		try:
			raw_links = self._read('__dependency_links__')
		except Exception:
			pass
		else:
			deps.dependency_links = list(pkg_resources.yield_lines(raw_links))
		return deps

	def _read(self, var_name):
		mod = ast.parse(self.script)
		node, = (
			node
			for node in mod.body
			if isinstance(node, ast.Assign)
			and len(node.targets) == 1
			and isinstance(node.targets[0], ast.Name)
			and node.targets[0].id == var_name
		)
		return ast.literal_eval(node.value)


def run(cmdline):
	"""
	Execute the script as if it had been invoked naturally.
	"""
	namespace = dict()
	filename = cmdline[0]
	namespace['__file__'] = filename
	namespace['__name__'] = '__main__'
	sys.argv[:] = cmdline

	open_ = getattr(tokenize, 'open', open)
	script = open_(filename).read()
	norm_script = script.replace('\\r\\n', '\\n')
	code = compile(norm_script, filename, 'exec')
	exec(code, namespace)
